#!/usr/local/bin/perl -w

sub usage {
    warn(
	 "Usage: combine.pl -dontround [<filename>...]+\n");
    exit 1;
}

sub arguments {
    $dontround = 0;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-dontround", 0, length $_)) {
		$dontround = 1;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}


sub abs {
    local($x) = @_;
    $x = 0-$x if ($x < 0);
    $x;
}

sub roundit { 
    local($value, $roundby) = @_;
    local($tmp) = abs($value) + $roundby/2;
    local($tmpround) =   int($tmp/$roundby)*$roundby;
    $tmpround = -$tmpround if ($value < 0);
    $tmpround;
}

sub roundtoXsigdigits { 
    local($value, $nsigdigits, $latex) = @_;
    local($roundby, $rem, $round_str);

    if ($dontround) {
	sprintf("%f", $value);
    }
    else {
	$avalue = $value;
	$avalue = 0-$value if ($value < 0);
	($avalue < 10000) || die("value $value too big for roundtoXsigdigits, fix!\n");
	($avalue > .001) || die("value $value too small for roundtoXsigdigits, fix!\n");
	if ($avalue > 1000) {
	    $roundby = 10;
	} elsif ($avalue > 88) {
	    $roundby = 1;
	} elsif ($avalue > 10) {
	    $roundby = .1;
	    $rem = 1;
	} elsif ($avalue > 1) {
	    $roundby = .01;
	    $rem = 2;
	} elsif ($avalue > .1) {
	    $roundby = .001;
	    $rem = 3;
	} elsif ($avalue > .01) {
	    $roundby = .0001;
	    $rem = 4;
	} elsif ($avalue > .001) {
	    $roundby = .00001;
	    $rem = 5;
	} 
	$roundby *= 10 if ($nsigdigits < 3);
	$roundby *= 10 if ($nsigdigits == 1);
	$rem-- if ($nsigdigits < 3);
	$rem-- if ($nsigdigits == 1);
	(($nsigdigits > 0) && ($nsigdigits < 4)) || die("$nsigdigits significant digits not supported yet, fix!\n");
	$round = &roundit($value, $roundby);
	if ($roundby >= 1) {
	    $round_str = sprintf("%d", $round);
	    $round_str .= "\\phantom{.00}" if ($latex);
	}
	else {
	    local($inte) = $nsigdigits - $rem;
	    $inte = 0 if ($inte < 0);
	    local($str) = sprintf("%%%d.%df", $inte, $nsigdigits-$inte);
	    $round_str = sprintf($str, $value);
	    $round_str .= "\\phantom{0}" if ($latex && ($inte == 2));
	}
	$round_str;
    }
}

sub ceilingit { 
    local($value, $roundby) = @_;
    local($tmp) = ($value-int($value/$roundby)*$roundby);
    local($valueround) = int($value/$roundby)*$roundby;
    $valueround += $roundby if ($tmp > 0);
    local($result) = $valueround;
    $result = $value if $dontround;
    $result;
}

sub ceilingtoXsigdigits { 
    local($value, $nsigdigits, $latex) = @_;
    local($roundby, $rem, $round_str);

    if ($dontround) {
	sprintf("%f", $value);
    }
    else {
	($value < 10000) || die("value $value too big for roundtoXsigdigits, fix!\n");
	($value > .01) || die("value $value too small for roundtoXsigdigits, fix!\n");
	if ($value > 1000) {
	    $roundby = 10;
	} elsif ($value > 100) {
	    $roundby = 1;
	} elsif ($value > 10) {
	    $roundby = .1;
	    $rem = 1;
	} elsif ($value > 1) {
	    $roundby = .01;
	    $rem = 2;
	} elsif ($value > .1) {
	    $roundby = .001;
	    $rem = 3;
	} elsif ($value > .01) {
	    $roundby = .0001;
	    $rem = 4;
	} 
	$roundby *= 10 if ($nsigdigits < 3);
	$roundby *= 10 if ($nsigdigits == 1);
	$rem-- if ($nsigdigits < 3);
	$rem-- if ($nsigdigits == 1);
	(($nsigdigits > 0) && ($nsigdigits < 4)) || die("$nsigdigits significant digits not supported yet, fix!\n");
	$round = &ceilingit($value, $roundby);
	if ($roundby >= 1) {
	    $round_str = sprintf("%d", $round);
	    $round_str .= "\\phantom{.0}" if ($latex);
	}
	else {
	    local($inte) = $nsigdigits - $rem;
	    $inte = 0 if ($inte < 0);
	    local($str) = sprintf("%%%d.%df", $inte, $nsigdigits-$inte);
	    $round_str = sprintf($str, $round);
#	    $round_str .= "\\phantom{0}" if ($latex && ($inte == 1));
	}
	$round_str;
    }
}

sub make_Tbatt_round { 
    local($Tbatt, $Tres, $latex) = @_;
    local($Tbatt_round);
    $Tbatt /= 3600;  # convert to hours
    if ($dontround) {
	$Tbatt_round = sprintf("%f", $Tbatt);
    }
    elsif ($Tres < 600) {
	$Tbatt_round = sprintf("%2.2f", $Tbatt);
    } elsif ($Tres == 600) {
	$Tbatt_round = sprintf("%3.1f", $Tbatt);
	$Tbatt_round .= "\\phantom{0}" if ($latex);
    } else {
	$Tbatt_round = sprintf("%3.1f", &roundit($Tbatt, .5));
	$Tbatt_round .= "\\phantom{0}" if ($latex);
    }
    $Tbatt_round;
}

sub make_Tbatt_round_days { 
    local($Tbatt, $Tres) = @_;
    local($Tbatt_round) = "";
    $Tbatt /= (3600*24);  # convert to days
    $Tbatt_round = sprintf("(%2.1f)", $Tbatt) if ($Tbatt > 10);
    $Tbatt_round = sprintf("(%1.1f)", $Tbatt) if (($Tbatt< 10) && ($Tbatt > 2));
    $Tbatt_round;
}

sub gen_rel_err { 
    local($value, $low, $high) = @_;
    local($maxAerr, $maxRerr);

    if (($value - $low) > $high - $value) {
	$maxAerr = $value - $low;
    }
    else {
	$maxAerr = $high - $value;
    }
    $maxRerr = $maxAerr/$value;
    $maxRerr;
}

sub bymyindex { 
    local($aa, $bb);
    ($aa, $junk) = split("#", $a);
    ($bb, $junk) = split("#", $b);
    $aa <=> $bb;
}

%results = ();
@listoftests = ();
@listofothertests = ();
@results_reusebat = ();
@results_nobatfault = ();

#@ltable = ();
@etables = ();

sub processit { # IN_FILEHANDLE 
    local($IN) = $_[0];

    printf "--------------------------------------------------------------------------------------------------------\n";
    printf "Individual data\n";
    while(<$IN>) {
	next if (/^Unexpected battery capacity/);
	($itsy, $type, $handle, $Pavg, $Rerr, $Tbatt, $Rerr_batt, $capacity, $Plowbnd, $Pupbnd, $Tres, $battery) = 
	    $_ =~ /^itsy #(\d+), type: (\S+), (\S+):\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+Tbatt\s+(\S+) s, R[eE]rr (\S+)%\s+\((\S+)\sWh\)\s+low:(\S+)\s+up:(\S+)\s+Tres:(\S+)\s+battery:(\S+)\s+/;
#	print "$itsy, $type, $handle, $Pavg, $Rerr, $Tbatt, $Rerr_batt, $capacity, $Plowbnd, $Pupbnd, $Tres, $battery\n";

	$Tbatt_round = &make_Tbatt_round($Tbatt, $Tres, 0);
	$Tbatt_round_latex = &make_Tbatt_round($Tbatt, $Tres, 1);

	printf 
	    "itsy #%d, bat %d, type: %s, %s:Pavg %s mW, RErr %s%%   Tbatt %s, RErr %s%%  (%s Wh)   low:%s up:%s Tres:%d\n", 
		$itsy, $battery, $type, $handle, 
		roundtoXsigdigits($Pavg, 3, 0), ceilingtoXsigdigits($Rerr, 2, 0), 
		$Tbatt_round, ceilingtoXsigdigits($Rerr_batt, 2, 0), 
		roundtoXsigdigits($capacity, 3, 0), 
		roundtoXsigdigits($Plowbnd, 3, 0), roundtoXsigdigits($Pupbnd, 3), $Tres;

	$prefix = $handle."-".$type;
	push(@listoftests, $prefix) if (! $results{$prefix});
	$tmp = sprintf("$itsy#%10.10f#%10.10f#%10.10f#%10.10f#%d#%d=", $Pavg, $Rerr, $Tbatt, $Rerr_batt, $Tres, $battery);
	$results{$prefix} .= $tmp;

	$tmp   = "";
	$tmp  .= "\\phantom{0}" if ($itsy < 10);
	$tmp  .= "$itsy";
	$tmp  .= "\\textsuperscript{\$*\$}" if ($type =~ /reusebat/);
	$tmp  .= "\\textsuperscript{\$\\dagger\$}" if ($type =~ /batfault/);
	$tmp1  = sprintf("%12s & ", $tmp);
	$tmp1 .= "0" if ($battery < 10);
	$tmp1 .= "$battery";
	$tmp   = sprintf("%2s ", $tmp1);
	
        printmorelatex(
	    $prefix, 
	    $tmp,
  	    roundtoXsigdigits($Pavg, 3, 1), ceilingtoXsigdigits($Rerr, 2, 1), 
  	    roundtoXsigdigits($Plowbnd, 3, 1), roundtoXsigdigits($Pupbnd, 3, 1), 
		$Tbatt_round_latex, ceilingtoXsigdigits($Rerr_batt, 2, 1), 
		roundtoXsigdigits($capacity, 3, 1), 
		);

    }
}


&arguments();

sub printit { #string, $pavg, $rerr, $tbatt, $rerr_batt, $capacity, $itsylist
    local($string, $pavg, $rerr, $tbatt, $rerr_batt, $capacity, $itsylist) = @_;
    printf 
	"%22s: Pavg: %5smW %4s%%  Tbatt: %5sh %4s%%  %5sWh (%s)\n",
	$string, $pavg, $rerr, $tbatt, $rerr_batt, $capacity, $itsylist;
}

%kerrprefixes = (
       "deepsleep",  "1#" . "deep sleep mode                                             & --  & ",
           "sleep",  "2#" . "sleep mode                                                  & --  & ",
         "sleepdc",  "3#" . "sleep mode, mem.\\ DC\\textsuperscript{\$*\$}               & --  & ",
        "sleeplcd",  "4#" . "sleep mode, static LCD image                                & --  & ",
      "idle59high",  "5#" . "system idle, idle mode                                     & \\phantom{0}59 & ",
     "idle206high",  "6#" . "system idle, idle mode                                     &            206 & ",
       "wav59high", " 7#" . "playing audio file (WAV)                                   & \\phantom{0}59 & ",
      "wav206high", " 8#" . "playing audio file (WAV)                                   &            206 & ",
       "tts74high", " 9#" . "text to speech\\textsuperscript{\$\\dagger\$}              & \\phantom{0}74 & ",
      "tts206high", "10#" . "text to speech\\textsuperscript{\$\\dagger\$}              &            206 & ",
     "dict206high", "11#" . "dictation,\\textsuperscript{\$\\ddagger\$} mem.\\ DC\\textsuperscript{\$*\$} & 206 & ",
     "mpeg206high", "12#" . "playing video file (MPEG-1) with audio                     &            206 & ",
       "idle59low", "13#" . "system idle, idle mode, low core voltage                   & \\phantom{0}59 & ",
        "tts74low", "14#" . "text to speech,\\textsuperscript{\$\\dagger\$} low core voltage & \\phantom{0}74 & ",
 );

%kerrsuffixes = (
       "deepsleep", "",
           "sleep", "",
         "sleepdc", "",
        "sleeplcd", "\\hline",
      "idle59high", "",
     "idle206high", "\\hline",
       "wav59high", "",
      "wav206high", "\\hline",
       "tts74high", "",
      "tts206high", "\\hline",
     "dict206high", "\\hline",
     "mpeg206high", "\\hline\\hline",
       "idle59low", "",
        "tts74low", "\\hline",
 );

%prefixes = (
     "mpeg206high", "15#" . " MPEG-1, 206         & ",
        "tts74low", "11#" . " DECtalk, 74, LV     & ",
       "tts74high", "12#" . " DECtalk, 74         & ",
      "tts206high", "13#" . " DECtalk, 206        & ",
       "wav59high",  "9#" . " WAV, 59             & ",
      "wav206high", "10#" . " WAV, 206            & ",
       "idle59low",  "5#" . " Idle, 59, LV        & ",
      "idle59high",  "6#" . " Idle, 59            & ",
     "idle133high",  "7#" . " Idle, 133           & ",
     "idle206high",  "8#" . " Idle, 206           & ",
     "dict206high", "14#" . " Dictation, DC, 206  & ",
       "deepsleep",  "1#" . " Deep sleep          & ", 
        "sleeplcd",  "4#" . " Sleep, LCD          & ",
         "sleepdc",  "3#" . " Sleep, DC           & ",
           "sleep",  "2#" . " Sleep               & ",
 );

%idles = (
     "mpeg206high", " 16 ",
        "tts74low", "\$<\$1",
       "tts74high", "\$<\$1",
      "tts206high", " 53 ",
       "wav59high", " 83 ",
      "wav206high", " 93 ",
       "idle59low", " 95 ",
      "idle59high", " 95 ",
     "idle133high", " 95 ",
     "idle206high", " 95 ",
     "dict206high", "\$<\$1",
       "deepsleep",  "\\multicolumn{1}{c||}{--}",
        "sleeplcd",  "\\multicolumn{1}{c||}{--}",
         "sleepdc",  "\\multicolumn{1}{c||}{--}",
           "sleep",  "\\multicolumn{1}{c||}{--}",
 );

%kidles = (
     "mpeg206high", " 16 ",
        "tts74low", "\$<\$1",
       "tts74high", "\$<\$1",
      "tts206high", " 53 ",
       "wav59high", " 83 ",
      "wav206high", " 93 ",
       "idle59low", " 95 ",
      "idle59high", " 95 ",
     "idle133high", " 95 ",
     "idle206high", " 95 ",
     "dict206high", "\$<\$1",
       "deepsleep",  " -- ",
        "sleeplcd",  " -- ",
         "sleepdc",  " -- ",
           "sleep",  " -- ",
 );

sub printitlatex { 
    local($string, $pavg, $rerr, $tbatt, $tbatt_days, $rerr_batt, $capacity, $nitsy) = @_;
    ($junk1, $junk2) = split(/-/, $string);
    $tmp = sprintf("%16s %d & %s & %17s & %16s & %17s & %5s & %16s \\\\\n",
		   $prefixes{$junk1}, $nitsy, $idles{$junk1},
		   $pavg, $rerr, $tbatt, $tbatt_days, $rerr_batt);
    ($junk, $real) = split("#", $tmp);
    $ltable[$junk-1] = $real;
}

sub printkerrlatex { 
    local($string, $pavg, $rerr, $tbatt, $rerr_batt, $capacity, $nitsy) = @_;
    ($junk1, $junk2) = split(/-/, $string);
    if ($kerrprefixes{$junk1}) {
	$tmp = sprintf("%16s %s & %17s\\phantom{0} & %17s\\phantom{0} & %16s \\\\%s \n",
		       $kerrprefixes{$junk1}, $kidles{$junk1},
		       $pavg, $tbatt, $capacity,
		       $kerrsuffixes{$junk1});
	($junk, $real) = split("#", $tmp);
	$ktable[$junk-1] = $real;
    }
}

sub printmorelatex { 
    local($string, $itsybatstr, $pavg, $rerr, $lbnd, $upbnd, $tbatt, $rerr_batt, $capacity) =  @_;
    $tmp = sprintf("%s@%10s & %17s & %16s & %17s & %17s & %17s & %16s & %5s \\\\\n",
		   $string,
		   $itsybatstr, $pavg, $rerr, $lbnd, $upbnd,
		   $tbatt, $rerr_batt, $capacity);
    ($junk1, $junk2) = split(/-/, $string);
    ($junk, $real) = split("#", $prefixes{$junk1});
    $etables[$junk-1] .= $tmp . "=";
}

sub doprint {
    local($string) = $_[0];
    local(*array) = $_[1];

    printf "--------------------------------------------------------------------------------------------------------\n";
    printf "$string\n";

    foreach $test (@listofothertests) {
	next unless $array{$test};
	$first = 1;
	@data = split(/=/, $array{$test});
	foreach $set (@data) {
	    ($type, $itsy, $Pavg, $Rerr, $Tbatt, $Rerr_batt, $Tres, $battery) = split("#", $set);
            $capacity = $Pavg*$Tbatt/3600/1000;
	    
            if (! $first) {										      
		# find matching one in main set
		@maindata = split(/=/, $results{"$test-fresh"});
		foreach $mainset (@maindata) {
		    local($mbattery,$mTres,$mRerr,$mRerr_batt);
		    ($mitsy, $mPavg, $mRerr, $mTbatt, $mRerr_batt, $mTres, $mbattery) = split("#", $mainset);
		    next unless ($mitsy == $itsy);

#		    $tmp = sprintf("%5s & %4s ", 
#				   roundtoXsigdigits(100*($mPavg-$Pavg)/$mPavg, 2, 1),
#				   roundtoXsigdigits(100*($mTbatt-$Tbatt)/$mTbatt, 2, 1));
		    $tmp = sprintf("%2.1f & %2.1f ", 
				   100*($mPavg-$Pavg)/$mPavg,
				   100*($mTbatt-$Tbatt)/$mTbatt);
		    $combined_results{$test} .= "& " if ($combined_results{$test});
		    $combined_results{$test} .= $tmp;
		    last;
		}
	    }
            $first = 0;

            printit(
	        $test."-".$type, 
  	        roundtoXsigdigits($Pavg, 3, 0), ceilingtoXsigdigits($Rerr, 2, 0),
                &make_Tbatt_round($Tbatt, $Tres, 0), ceilingtoXsigdigits($Rerr_batt, 2, 0), 
                roundtoXsigdigits($capacity, 3, 0),
                $itsy
		    );
        }
        printf "\n";
    }
}


FILE:

if ($#ARGV < 0) {
    &usage();
}
while ($#ARGV >= 0) {
    $infile = shift @ARGV;
    open(FILE, $infile) || (die "Can't open $infile: $!\n");
    
    processit(FILE);
    close(FILE);
    
    printf "--------------------------------------------------------------------------------------------------------\n";
    printf "Aggregated data\n";
    foreach $test (@listoftests) {
	@data = split(/=/, $results{$test});
	$count = 0;
	$aPavg = 0;
	$lAerr = 100000;
	$hAerr = 0;
	$aTbatt = 0;
	$lAerr_batt = 100000000;
	$hAerr_batt = 0;
	$itsylist = "";
	$nitsy = 0;
	foreach $set (@data) {
	    ($itsy, $Pavg, $Rerr, $Tbatt, $Rerr_batt, $Tres, $battery) = split("#", $set);

	    ($handle, $type) = split(/-/, $test);
            if ($itsy == 17) {
		push(@listofothertests, $handle) if (! $results_reusebat{$handle});
		$results_nobatfault{$handle} .= $type."#".$set."=" if (($type =~ /fresh/) || ($type =~ /nobatfault/));
		$results_reusebat{$handle}   .= $type."#".$set."=" if (($type =~ /fresh/) || ($type =~ /reusebat/));
	    }
            next if (($type =~ /nobatfault/) || ($type =~ /reusebat/));

            $aPavg += $Pavg;
            $Aerr = ($Rerr*$Pavg)/100;
            $current_lAerr = $Pavg - $Aerr;
            $current_hAerr = $Pavg + $Aerr;
            ($current_lAerr < $lAerr) && ($lAerr = $current_lAerr); 
	    ($current_hAerr > $hAerr) && ($hAerr = $current_hAerr); 

            $aTbatt += $Tbatt;
            $Aerr_batt = ($Rerr_batt*$Tbatt)/100;
            $current_lAerr_batt = $Tbatt - $Aerr_batt;
            $current_hAerr_batt = $Tbatt + $Aerr_batt;
            ($current_lAerr_batt < $lAerr_batt) && ($lAerr_batt = $current_lAerr_batt); 
	    ($current_hAerr_batt > $hAerr_batt) && ($hAerr_batt = $current_hAerr_batt); 

            $count += 1;
	    $itsylist .= "," if $itsylist;
	    $itsylist .= $itsy;
            $nitsy += 1;
        }
        next if ($count == 0);
        $aPavg = $aPavg/$count;
        $aTbatt = $aTbatt/$count;

	$maxRerr = gen_rel_err($aPavg, $lAerr, $hAerr);
	$maxRerr_batt = gen_rel_err($aTbatt, $lAerr_batt, $hAerr_batt);

        $capacity = $aPavg*$aTbatt/3600/1000;

        printit(
	    $test, 
	    roundtoXsigdigits($aPavg, 3, 0), ceilingtoXsigdigits($maxRerr*100, 2, 0),
	    &make_Tbatt_round($aTbatt, $Tres, 0), ceilingtoXsigdigits($maxRerr_batt*100, 2, 0), 
	    roundtoXsigdigits($capacity, 3, 0),
	    $itsylist,
		);
        printitlatex(
	    $test, 
	    roundtoXsigdigits($aPavg, 3, 1), ceilingtoXsigdigits($maxRerr*100, 2, 1),
	    &make_Tbatt_round($aTbatt, $Tres, 1), &make_Tbatt_round_days($aTbatt, $Tres), ceilingtoXsigdigits($maxRerr_batt*100, 2, 1), 
	    roundtoXsigdigits($capacity, 3, 0),
	    $nitsy, 
		);
        printkerrlatex(
	    $test, 
	    roundtoXsigdigits($aPavg, 3, 1), ceilingtoXsigdigits($maxRerr*100, 2, 1),
	    &make_Tbatt_round($aTbatt, $Tres, 1), ceilingtoXsigdigits($maxRerr_batt*100, 2, 1), 
	    roundtoXsigdigits($capacity, 3, 0),
	    $nitsy, 
		);
    }
	doprint("reuse battery experiments", *results_reusebat);
	doprint("no batfault experiments", *results_nobatfault);

    printf "--------------------------------------------------------------------------------------------------------\n";

    $filename = "other.tex";
    open(BFILE, "> $filename") || (die "Can't open $filename: $!\n");
    select(BFILE);
    foreach $test (@listofothertests) {
	next unless $combined_results{$test};
	($order, $prefix) = split("#", $prefixes{$test});
	$tmp = sprintf("%16s %s \\\\\n", $prefix, $combined_results{$test});
	$otable[$order-4] = $tmp;
    }
    foreach $set (@otable) {
        printf "$set" if $set;
    }
    close(BFILE);

    open(BFILE, "> globres.tex") || (die "Can't open globres.tex: $!\n");
    select(BFILE);
    foreach $set (@ltable) {
        printf "$set";
    }
    close(BFILE);

    open(BFILE, "> xieee.tex") || (die "Can't open xieee.tex: $!\n");
    select(BFILE);
    foreach $set (@ktable) {
        printf "$set";
    }
    close(BFILE);

    foreach $set (@etables) {
	$first = 1;
	@data = split(/=/, $set);
	foreach $line (@data) {
	    ($prefix, $tab) = split(/@/, $line);
	    ($handle, $type) = split(/-/, $prefix);
	    if ($first) {
		open(BFILE, "> $handle.tex") || (die "Can't open $handle.tex: $!\n");
		select(BFILE);
		$first = 0;
	    }
	    printf "\\hline%%---------------------------------------------------------------\n" if ($type =~ /reusebat/);
	    printf "$tab";
	}
	close(BFILE);
    }

}

